import os
import sys
import importlib


class ChangeDirectoryAndPath:
    """Context manager to change the current working directory and sys.path."""

    def __init__(self, module_path):
        self.module_path = module_path
        self.old_path = None

    def __enter__(self):
        self.old_path = os.getcwd()
        sys.path.insert(0, self.module_path)
        os.chdir(self.module_path)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        sys.path.remove(self.module_path)
        os.chdir(self.old_path)
        

class ModuleManager:
    """Context manager for handling module imports and sys.modules state."""

    def __init__(self, module_path):
        self.module_path = module_path
        self.original_modules = sys.modules.copy()

    def __enter__(self):
        """Enter the runtime context related to this object."""
        self.change_dir = ChangeDirectoryAndPath(self.module_path)
        self.change_dir.__enter__()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """Exit the runtime context related to this object."""
        self.change_dir.__exit__(exc_type, exc_value, traceback)
        self.cleanup_modules()

    def cleanup_modules(self):
        """Restore the original sys.modules state."""
        importlib.invalidate_caches()
        for key in list(sys.modules.keys()):
            if key not in self.original_modules:
                del sys.modules[key]
                

def wrapped_func(item, module_path):
    original_tool_function = item.func

    def _wrapped(*args, **kwargs):
        print(f"调用了 {item.name}")
        with ChangeDirectoryAndPath(module_path):
            result = original_tool_function(*args, **kwargs)
        print(f"{item.name} 调用完毕,结果为:", result)
        return result

    return _wrapped





import json
from pycoze import utils
import json
import inspect


params_file = utils.read_arg("params_file", True)
params = None
try:
    with open(params_file, "r", encoding="utf-8") as f:
        params = json.load(f)
except Exception as e:
    print(e)


def get_ui():
    if params is None or "uiData" not in params or "uiRecord" not in params["uiData"]:
        return {}
    stack = inspect.stack()
    stack_files = list(reversed([s.filename.replace("\\", "/") for s in stack]))
    match_ui_file = None
    for f in stack_files:
        for v in params["uiData"]["uiRecord"]:
            if v == f:
                match_ui_file = v
                break
        if match_ui_file is not None:
            break

    if match_ui_file is None:
        raise Exception("uiData not found for file")

    by = params["uiData"]["by"]

    for i in range(10):
        try:
            if by == "file":
                json_file = params["uiData"]["uiRecord"][match_ui_file]
                with open(json_file, "r", encoding="utf-8") as f:
                    return json.load(f)
            else:
                assert by == "workflow"
                return params["uiData"]["uiRecord"][match_ui_file]
        except Exception as e:
            if i == 9:
                raise e
